package org.cometd.javascript.jquery;

import java.util.Arrays;

import org.cometd.javascript.Latch;

/**
 * @version $Revision: 1224 $ $Date: 2010-05-27 15:51:02 +0200 (Thu, 27 May 2010) $
 */
public class CometdTransportTest extends AbstractCometdJQueryTest
{
    public void testTransport() throws Exception
    {
        defineClass(Latch.class);
        evaluateScript("$.cometd.configure({url: '" + cometdURL + "', logLevel: 'debug'});");
        Object[] transportTypes = (Object[])jsToJava(evaluateScript("$.cometd.getTransportTypes()"));
        // The spec requires at least these 2 transports
        assertTrue(Arrays.asList(transportTypes).contains("long-polling"));
        assertTrue(Arrays.asList(transportTypes).contains("callback-polling"));

        // Remove all transports
        evaluateScript("" +
                "var types = $.cometd.getTransportTypes();" +
                "for (var i = 0; i < types.length; ++i)" +
                "{" +
                "   $.cometd.unregisterTransport(types[i]);" +
                "};");

        String localTransport =
                "var readyLatch = new Latch(1);" +
                "function LocalTransport()" +
                "{" +
                "    var _super = new org.cometd.RequestTransport();" +
                "    var that = org.cometd.Transport.derive(_super);" +
                "    var _sends = 0;" +
                "" +
                "    that.getSends = function() { return _sends; };" +
                "" +
                "    that.accept = function(version, crossDomain)" +
                "    {" +
                "        return true;" +
                "    };" +
                "" +
                "    that.transportSend = function(envelope, request)" +
                "    {" +
                "        ++_sends;" +
                "        var response;" +
                "        var timeout;" +
                "        switch (_sends)" +
                "        {" +
                "            case 1:" +
                "                response = '[{" +
                "                    \"successful\":true," +
                "                    \"channel\":\"/meta/handshake\"," +
                "                    \"clientId\":\"dmigjcjnakuysa9j29\"," +
                "                    \"id\":\"1\"," +
                "                    \"minimumVersion\":\"0.9\"," +
                "                    \"version\":\"1.0\"," +
                "                    \"supportedConnectionTypes\":[\"long-polling\",\"callback-polling\"]," +
                "                    \"advice\":{\"reconnect\":\"retry\",\"interval\":0,\"timeout\":5000}" +
                "                }]';" +
                "                timeout = 0;" +
                "                break;" +
                "            case 2:" +
                "                response = '[{" +
                "                    \"successful\":true," +
                "                    \"channel\":\"/meta/connect\"," +
                "                    \"id\":\"2\"," +
                "                    \"advice\":{\"reconnect\":\"retry\",\"interval\":0,\"timeout\":5000}" +
                "                }]';" +
                "                timeout = 0;" +
                "                break;" +
                "            case 3:" +
                "                response = '[{" +
                "                    \"successful\":true," +
                "                    \"channel\":\"/meta/connect\"," +
                "                    \"id\":\"2\"," +
                "                    \"advice\":{\"reconnect\":\"retry\",\"interval\":0,\"timeout\":5000}" +
                "                }]';" +
                "                timeout = 5000;" +
                "                readyLatch.countDown();" +
                "                break;" +
                "            case 4:" +
                "                response = '[{" +
                "                    \"successful\":true," +
                "                    \"channel\":\"/meta/disconnect\"," +
                "                    \"id\":\"3\"" +
                "                }]';" +
                "                timeout = 0;" +
                "                break;" +
                "            default:" +
                "                throw 'Test Error';" +
                "        }" +
                "" +
                "        /* Respond asynchronously */" +
                "        var self = this;" +
                "        setTimeout(function()" +
                "        {" +
                "            self.transportSuccess(envelope, request, self.convertToMessages(response));" +
                "        }, timeout);" +
                "    };" +
                "" +
                "    return that;" +
                "};";

        evaluateScript(localTransport);

        evaluateScript("var localTransport = new LocalTransport();");
        // The server does not support a 'local' transport, so use 'long-polling'
        assertTrue((Boolean)evaluateScript("$.cometd.registerTransport('long-polling', localTransport);"));

        Latch readyLatch = get("readyLatch");
        evaluateScript("$.cometd.handshake();");
        assertTrue(readyLatch.await(1000));

        assertEquals(3, ((Number)evaluateScript("localTransport.getSends();")).intValue());
        assertEquals("connected", evaluateScript("$.cometd.getStatus();"));

        readyLatch.reset(1);
        evaluateScript("$.cometd.addListener('/meta/disconnect', readyLatch, 'countDown');");
        evaluateScript("$.cometd.disconnect();");
        assertTrue(readyLatch.await(1000));

        assertEquals(4, ((Number)evaluateScript("localTransport.getSends();")).intValue());
        assertEquals("disconnected", evaluateScript("$.cometd.getStatus();"));
    }
}
